import argparse
import os
import logging
import shutil
from pathlib import Path
from typing import NamedTuple
import subprocess
import uuid
from multiprocessing import Pool

logger = logging.getLogger(__name__)


def convert_audio_file(args):
    cmd, output_tmp, output_path = args

    subprocess.run(cmd, shell=True)
    os.makedirs(output_path.parent, exist_ok=True)
    shutil.move(output_tmp, output_path)


class ConversionConfig(NamedTuple):
    input_dir: Path
    output_dir: Path
    output_fmt: str
    input_fmt: str


def _get_converted_path(input_path: Path, config: ConversionConfig):
    """
    Return the path you would obtain if you moved the file in the input
    path to the output directory in the config and changed its extension.
    """

    relative_path = input_path.relative_to(config.input_dir)
    output_filename = str(input_path.stem) + f".{config.output_fmt}"
    output_filename.replace("'", "")
    output_path = config.output_dir / relative_path.parent / output_filename
    return output_path


def get_files_recursive(search_path: Path, extension: str):
    return list(search_path.rglob(f"*.{extension}"))


def convert(config: ConversionConfig):

    logger.info("Converting files in %s", config.input_dir)
    logger.info("Writing output to: %s", config.output_dir)

    os.makedirs(config.output_dir, exist_ok=True)

    input_files = get_files_recursive(
        config.input_dir.resolve(), config.input_fmt
    )
    output_files = []
    for input_file in input_files:
        candidate_output = _get_converted_path(input_file)
        if not candidate_output.exists():
            output_files.append(candidate_output)
    print(output_files)

    tasks = []
    for idx, (input_path, output_path) in enumerate(zip(input_files, output_files)):

        logger.info("Converting file %d of %d", idx, len(input_files))

        output_tmp = self.output_dir / (str(uuid.uuid4()) + f".{self.output_fmt}")
        cmd = f"ffmpeg -i '{input_path}' -ab 320k -map_metadata 0 -id3v2_version 3 '{output_tmp}'"
        tasks.append((cmd, output_tmp, output_path))

    with Pool(10) as p:
        p.map(convert_audio_file, tasks)


def move(input_dir: Path, output_dir: Path):

    logger.info("Moving files in %s to %s", input_dir, output_dir)
    os.makedirs(output_dir, exist_ok=True)

    mp3_files = list(input_dir.resolve().rglob("*.mp3"))
    for idx, path in enumerate(mp3_files):
        logger.info("Moving file %d of %d", idx, len(mp3_files))
        relative_path = path.relative_to(input_dir)
        output_path = output_dir / relative_path
        os.makedirs(output_path.parent, exist_ok=True)
        shutil.move(path, output_path)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--input_dir",
        type=Path,
        default=Path(),
        help="Directory with input files for conversion.",
    )

    parser.add_argument(
        "--output_dir", type=Path, default=Path(), help="Directory for converted files"
    )

    args = parser.parse_args()

    logging.basicConfig()
    logging.getLogger().setLevel(logging.INFO)

    converter = AudioFileConverter(args.input_dir.resolve(), args.output_dir.resolve())
    converter.convert()
    # move(args.input_dir.resolve(), args.output_dir.resolve())
